@@ -51,8 +51,8 @@ must conform to the Nmap Security Scanner licence
 
 
 __author__ = 'Alexandre Norman (norman@xael.org)'
-__version__ = '0.6.0'
-__last_modification__ = '2016.03.15'
+__version__ = '0.6.1'
+__last_modification__ = '2016.07.29'
 
 
 import csv
@@ -246,14 +246,15 @@ class PortScanner(object):
         # keeps on trucking.
 
         nmap_err_keep_trace = []
+        nmap_warn_keep_trace = []
         if len(nmap_err) > 0:
-            regex_warning = re.compile('^Warning: .*')
+            regex_warning = re.compile('^Warning: .*', re.IGNORECASE)
             for line in nmap_err.split(os.linesep):
                 if len(line) > 0:
                     rgw = regex_warning.search(line)
                     if rgw is not None:
-                        sys.stderr.write(line+os.linesep)
-                        pass
+                        # sys.stderr.write(line+os.linesep)
+                        nmap_warn_keep_trace.append(line+os.linesep)
                     else:
                         # raise PortScannerError(nmap_err)
                         nmap_err_keep_trace.append(nmap_err)
@@ -261,11 +262,12 @@ class PortScanner(object):
         return self.analyse_nmap_xml_scan(
             nmap_xml_output=self._nmap_last_output,
             nmap_err=nmap_err,
-            nmap_err_keep_trace=nmap_err_keep_trace
+            nmap_err_keep_trace=nmap_err_keep_trace,
+            nmap_warn_keep_trace=nmap_warn_keep_trace
         )
 
 
-    def analyse_nmap_xml_scan(self, nmap_xml_output=None, nmap_err='', nmap_err_keep_trace=''):
+    def analyse_nmap_xml_scan(self, nmap_xml_output=None, nmap_err='', nmap_err_keep_trace='', nmap_warn_keep_trace=''):
         """
         Analyses NMAP xml scan ouput
 
@@ -339,6 +341,10 @@ class PortScanner(object):
         if len(nmap_err_keep_trace)>0:
             scan_result['nmap']['scaninfo']['error'] = nmap_err_keep_trace
 
+        # if there was a warning
+        if len(nmap_warn_keep_trace)>0:
+            scan_result['nmap']['scaninfo']['warning'] = nmap_warn_keep_trace
+
         # info about scan
         for dsci in dom.findall('scaninfo'):
             scan_result['nmap']['scaninfo'][dsci.get('protocol')] = {                
@@ -366,11 +372,18 @@ class PortScanner(object):
                 host = dhost.find('address').get('addr')
                 
             hostnames = []
-            for dhostname in dhost.findall('hostnames/hostname'):
-                hostnames.append({
-                    'name':dhostname.get('name'),
-                    'type':dhostname.get('type'),
+            if len(dhost.findall('hostnames/hostname')) > 0:
+                for dhostname in dhost.findall('hostnames/hostname'):
+                    hostnames.append({
+                        'name':dhostname.get('name'),
+                        'type':dhostname.get('type'),
                     })
+            else:
+                hostnames.append({
+                    'name':'',
+                    'type':'',
+                })
+
             scan_result['scan'][host] = PortScannerHostDict({'hostnames': hostnames})
 
             scan_result['scan'][host]['addresses'] = address_block
@@ -690,7 +703,8 @@ def __scan_progressive__(self, hosts, ports, arguments, callback, sudo):
         try:
             scan_data = self._nm.scan(host, ports, arguments, sudo)
         except PortScannerError:
-            pass
+            scan_data = None
+
         if callback is not None:
             callback(host, scan_data)
     return
@@ -737,7 +751,7 @@ class PortScannerAsync(object):
         """
         Scan given hosts in a separate process and return host by host result using callback function
 
-        PortScannerError exception from standard nmap is catched and you won't know about it
+        PortScannerError exception from standard nmap is catched and you won't know about but get None as scan_data
 
         :param hosts: string for hosts as nmap use it 'scanme.nmap.org' or '198.116.0-255.1-127' or '216.163.128.20/20'
         :param ports: string for ports as nmap use it '22,53,110,143-4564'
@@ -849,26 +863,21 @@ class PortScannerYield(PortScannerAsync):
             try:
                 scan_data = self._nm.scan(host, ports, arguments, sudo)
             except PortScannerError:
-                pass
+                scan_data = None
             yield (host, scan_data)
         return
 
-
-
     def stop(self):
         pass
 
     def wait(self, timeout=None):
         pass
-    
 
     def still_scanning(self):
         pass
 
-    
 
 ############################################################################
-    
 
 
 class PortScannerHostDict(dict):
@@ -894,7 +903,7 @@ class PortScannerHostDict(dict):
             if h['type'] == 'user':
                 return h['name']
         else:
-            if len(self['hostnames']) > 0 and 'name' in self['hostnames']:
+            if len(self['hostnames']) > 0 and 'name' in self['hostnames'][0]:
                 return self['hostnames'][0]['name']
             else:
                 return ''
